#include <iostream> // Pour afficher le résultat
#include <random> // Pour simuler des variables aléatoire
#include <cmath> // Pour la fonction sqrt

int const N = 100; // Le nombre de pas de discrétisation
int const MC = 100000; // Le nombre de simulations indépendantes

// Une fonction qui simule un pont Brownien et regarde si la trajectoire dépasse ou non la valeur b.
bool depasse(double b, std::mt19937 &gen)
{
  double x = 0;
  std::normal_distribution<double> X(0,1);
  
  for (int k=0; k<N; k++)
	{
	  x   =   x*(N-k-1.)/(N-k)   +  sqrt((N-k-1.)/(N*(N-k))) * X(gen);
	  if (x>b)// Dès que l'on dépasse la valeur b, on sait que l'évènemen s'est réalisé.
		return true;
	}
  // Si après avoir simuler toute la trajectoire, on n'a pas dépassé, l'évènement ne s'est pas réalisé.
  return false;
}

// Une fonction qui approche p(b) en appelant MC fois la fonction précédente.
double p(double b, std::mt19937 &gen)
{
  int compte = 0;
  for (int k=0; k<MC; k++)
	if (depasse(b,gen))
	  compte += 1;
        
  return compte/(double)MC;
}


int main (void)
{
  std::mt19937 gen(137);

  for (double b=1; b<=3; b+=1) // On affiche le résultat pour les valeurs b=1, 2 et 3.
	{
	std::cout << "p(" << b << ") ~ " << p(b,gen) << " (valeur théorique : " << exp(-2*b*b) << ")\n";
	}
  
  
  return 0;
}


